/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * robotviz 0.1                                                          *
 * Copyright (C) 2013  Holger Jost                                       *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "RobotViz.h"

vector<tuple<int,int,string>>        RobotViz::barrierList;
vector<tuple<int,int,int,int>>       RobotViz::positionList;
vector<tuple<string,int,int>>        RobotViz::targetList;
vector<tuple<int,int,string,int>>    RobotViz::moveList;

int RobotViz::width = 1;
int RobotViz::height = 1;

int RobotViz::steps = 1;

bool RobotViz::boardOnly = false;

int RobotViz::wOffset = 0;
int RobotViz::hOffset = 0;
double RobotViz::origRatio = 1.0;
double RobotViz::currRatio = 1.0;

int RobotViz::tileSize = 50;
int RobotViz::screenWidth;
int RobotViz::screenHeight;
SDL_Surface *RobotViz::screen;
SDL_Surface *RobotViz::board;
SDL_Surface *RobotViz::tile[3];
SDL_Surface *RobotViz::robot[4];
SDL_Surface *RobotViz::wallNorth, *RobotViz::wallWest;
SDL_Surface *RobotViz::item[17];
SDL_Surface *RobotViz::arrow[16];

list<pair<int, int>> RobotViz::wallsNorth;
list<pair<int, int>> RobotViz::wallsWest;

Item RobotViz::targetItem;

int RobotViz::numRobots = 4;
vector<pair<int, int>> RobotViz::robotPositions[4];
unsigned RobotViz::step = 0;

Item::Item() {
	x = 0;
	y = 0;
	symbol = random_robot;
}

Item::Item(int x, int y, Symbol symbol): x(x), y(y), symbol(symbol) {}

void RobotViz::parseDim(string input) {

	string startString;
	string endString;
	size_t startPos;
	size_t endPos;
	size_t pStartPos;
	size_t pEndPos;

	startString = "dim(";
	endString = ")";
	startPos = 0;
	endPos = 0;

	while(1) {
		startPos = input.find(startString,endPos);
		if(startPos==string::npos) break;
		endPos = input.find(endString, startPos);
		//cout << input.substr(startPos,endPos-startPos+1) << endl;
		string predString = input.substr(startPos,endPos-startPos+1);
		int dim;
		pStartPos = startPos + startString.length();
		pEndPos = input.find(")",pStartPos);
		dim = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		if(dim > width || dim > height) {
			width = dim;
			height = dim;
		}
	}
}

void RobotViz::parseBarrier(string input) {

	string startString;
	string endString;
	size_t startPos;
	size_t endPos;
	size_t pStartPos;
	size_t pEndPos;

	startString = "barrier(";
	endString = ")";
	startPos = 0;
	endPos = 0;

	while(1) {
		startPos = input.find(startString,endPos);
		if(startPos==string::npos) break;
		endPos = input.find(endString, startPos);
		//cout << input.substr(startPos,endPos-startPos+1) << endl;
		string predString = input.substr(startPos,endPos-startPos+1);
		int i;
		int j;
		int hor;
		int ver;
		string dir;
		pStartPos = startPos + startString.length();
		pEndPos = input.find(",",pStartPos);
		i = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		pStartPos = pEndPos + 1;
		pEndPos = input.find(",",pStartPos);
		j = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		pStartPos = pEndPos + 1;
		pEndPos = input.find(",",pStartPos);
		hor = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		pStartPos = pEndPos + 1;
		pEndPos = input.find(")",pStartPos);
		ver = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		if(hor == 1 && ver == 0) {
			i++;
			dir = "west";
		}
		else if(hor == -1 && ver == 0) dir = "west";
		else if(hor == 0 && ver == 1) {
			j++;
			dir = "north";
		}
		else if(hor == 0 && ver == -1) dir = "north";
		else {
			cerr << "Error: Barrier has unknown direction!" << endl;
			exit(1);
		}
		RobotViz::barrierList.push_back(tuple<int,int,string>(i,j,dir));
	}	
}

void RobotViz::parsePosition(string input) {

	string startString;
	string endString;
	size_t startPos;
	size_t endPos;
	size_t pStartPos;
	size_t pEndPos;

	startString = "position(";
	endString = ")";
	startPos = 0;
	endPos = 0;

	while(1) {
		startPos = input.find(startString,endPos);
		if(startPos==string::npos) break;
		endPos = input.find(endString, startPos);
		//cout << input.substr(startPos,endPos-startPos+1) << endl;
		string predString = input.substr(startPos,endPos-startPos+1);
		string colorString;
		int col;
		int x;
		int y;
		int time;
		pStartPos = startPos + startString.length();
		pEndPos = input.find(",",pStartPos);
		colorString = input.substr(pStartPos,pEndPos-pStartPos);
		pStartPos = pEndPos + 1;
		pEndPos = input.find(",",pStartPos);
		x = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		pStartPos = pEndPos + 1;
		pEndPos = input.find(")",pStartPos);
		y = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		time = 0;
		if(colorString.compare("red") == 0) {
			col = 0;
		} else if(colorString.compare("green") == 0) {
			col = 1;
		} else if(colorString.compare("blue") == 0) {
			col = 2;
		} else if(colorString.compare("yellow") == 0) {
			col = 3;
		} else {
			cerr << "Error: Unknown color!" << endl;
			exit(1);
		}
		RobotViz::positionList.push_back(tuple<int,int,int,int>(col,x,y,time));
	}
}

void RobotViz::parseTarget(string input) {

	string startString;
	string endString;
	size_t startPos;
	size_t endPos;
	size_t pStartPos;
	size_t pEndPos;

	startString = "target(";
	endString = ")";
	startPos = 0;
	endPos = 0;

	while(1) {
		startPos = input.find(startString,endPos);
		if(startPos==string::npos) break;
		endPos = input.find(endString, startPos);
		//cout << input.substr(startPos,endPos-startPos+1) << endl;
		string predString = input.substr(startPos,endPos-startPos+1);
		string col;
		int i;
		int j;
		pStartPos = startPos + startString.length();
		pEndPos = input.find(",",pStartPos);
		col = input.substr(pStartPos,pEndPos-pStartPos);
		pStartPos = pEndPos + 1;
		pEndPos = input.find(",",pStartPos);
		i = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		pStartPos = pEndPos + 1;
		pEndPos = input.find(")",pStartPos);
		j = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		targetList.push_back(tuple<string,int,int>(col,i,j));
	}	
	//cout << RobotViz::targetList.size() << endl;
	if(targetList.size() <= 0) {
		cout << "Warning: Couldn't find a target!" << endl;
		boardOnly = true;
	} else {
		if(targetList.size() > 1) cout << "Warning: More than one target found!" << endl;
		int symb;
		if(get<0>(targetList.at(0)).compare("red") == 0) {
			symb = sun_red;
		} else if(get<0>(targetList.at(0)).compare("green") == 0) {
			symb = sun_green;
		} else if(get<0>(targetList.at(0)).compare("blue") == 0) {
			symb = sun_blue;
		} else if(get<0>(targetList.at(0)).compare("yellow") == 0) {
			symb = sun_yellow;
		} else {
			cerr << "Error: Target has unknown color!" << endl;
			exit(1);
		}
		symb += 4 * (rand() % 4);
		targetItem = Item(get<1>(targetList.at(0)),get<2>(targetList.at(0)),(Symbol)symb);
	}
}

void RobotViz::parseMove(string input) {

	string startString;
	string endString;
	size_t startPos;
	size_t endPos;
	size_t pStartPos;
	size_t pEndPos;

	startString = "move(";
	endString = ")";
	startPos = 0;
	endPos = 0;

	while(1) {
		startPos = input.find(startString,endPos);
		if(startPos==string::npos) break;
		endPos = input.find(endString, startPos);
		//cout << input.substr(startPos,endPos-startPos+1) << endl;
		string predString = input.substr(startPos,endPos-startPos+1);
		string colorString;
		int col;
		int hor;
		int ver;
		string dir;
		int time;
		pStartPos = startPos + startString.length();
		pEndPos = input.find(",",pStartPos);
		colorString = input.substr(pStartPos,pEndPos-pStartPos);
		pStartPos = pEndPos + 1;
		pEndPos = input.find(",",pStartPos);
		hor = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		pStartPos = pEndPos + 1;
		pEndPos = input.find(",",pStartPos);
		ver = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		pStartPos = pEndPos + 1;
		pEndPos = input.find(")",pStartPos);
		time = atoi(input.substr(pStartPos,pEndPos-pStartPos).c_str());
		//targetList.push_back(tuple<string,int,int>(col,i,j));

		int arrow;
		if(colorString.compare("red") == 0) {
			arrow = 0;
			col = 0;
		} else if(colorString.compare("green") == 0) {
			arrow = 1;
			col = 1;
		} else if(colorString.compare("blue") == 0) {
			arrow = 2;
			col = 2;
		} else if(colorString.compare("yellow") == 0) {
			arrow = 3;
			col = 3;
		} else {
			cerr << "Error: Robots must be either red, green, blue or yellow!" << endl;
			exit(1);
		}
		// north
		if(hor == 0 && ver == -1) {
			arrow += 0;
			dir = "north";
		// east
		} else if(hor == 1 && ver == 0) {
			arrow += 4;
			dir = "east";
		// south
		} else if(hor == 0 && ver == 1) {
			arrow += 8;
			dir = "south";
		// west
		} else if(hor == -1 && ver == 0) {
			arrow += 12;
			dir = "west";
		} else {
			cerr << "Error: Direction must be either north, east, south or west!" << endl;
		}
		RobotViz::moveList.push_back(tuple<int,int,string,int>(col,arrow,dir,time));
	}
	// sort
	sort(moveList.begin(), moveList.end(), sortMoveTuple);
	//cout << RobotViz::moveList.size() << endl;
}

/* We only consider north or west walls */
void RobotViz::constructBoard() {
	//cout << barrierList.size() << endl;
	for(unsigned i=0; i<barrierList.size(); i++) {
		int x;
		int y;
		string dir;
		x = get<0>(barrierList.at(i));
		y = get<1>(barrierList.at(i));
		dir = get<2>(barrierList.at(i));
		if(dir.compare("north") == 0) {
			wallsNorth.push_back(pair<int, int>(x,y));
		} else if(dir.compare("west") == 0) {
			wallsWest.push_back(pair<int, int>(x,y));
		}
		//cout << wallsWest.size() << endl;
	}
	return;
}

bool RobotViz::sortMoveTuple(const tuple<int,int,string,int> &lhs, const tuple<int,int,string,int> &rhs) {
	return get<3>(lhs) < get<3>(rhs);
}

void RobotViz::calcPositions() {
	for(unsigned i=0; i<positionList.size(); i++) {
		robotPositions[get<0>(positionList.at(i))].push_back(pair<int,int>(get<1>(positionList.at(i)),get<2>(positionList.at(i))));
	}
	vector<pair<int, int>> tempPositions[numRobots];
	for(unsigned i=0; i<moveList.size(); i++) {

		for(unsigned j=0; j<numRobots; j++) {
			if(get<0>(moveList.at(i)) != j) {
				tempPositions[j].push_back(pair<int,int>(get<0>(robotPositions[j].back()),get<1>(robotPositions[j].back())));
			} else {
				// TODO: calc new position
				int x = get<0>(robotPositions[j].back());
				int y = get<1>(robotPositions[j].back());
				if(get<2>(moveList.at(i)).compare("north") == 0) {
					bool stopRobot = false;
					while(1) {
						// check for north walls
						for(list<pair<int, int>>::iterator k = wallsNorth.begin(); k != wallsNorth.end(); k++) {
							if(k->first == x && k->second == y) stopRobot = true;
						}
						// check for other robots
						for(unsigned l=0; l<numRobots; l++) {
							if(l != j && get<0>(robotPositions[l].back()) == x && get<1>(robotPositions[l].back()) == y-1) stopRobot = true;
						}
						// check for end of board
						if(y-1 < 1) stopRobot = true;
						// check for break
						if(!stopRobot) y -= 1;
						else break;
					}
				} else if(get<2>(moveList.at(i)).compare("east") == 0) {
					bool stopRobot = false;
					while(1) {
						// check for west walls
						for(list<pair<int, int>>::iterator k = wallsWest.begin(); k != wallsWest.end(); k++) {
							if(k->first == x+1 && k->second == y) stopRobot = true;
						}
						// check for other robots
						for(unsigned l=0; l<numRobots; l++) {
							if(l != j && get<0>(robotPositions[l].back()) == x+1 && get<1>(robotPositions[l].back()) == y) stopRobot = true;
						}
						// check for end of board
						if(x+1 > width) stopRobot = true;
						// check for break
						if(!stopRobot) x += 1;
						else break;
					}
				} else if(get<2>(moveList.at(i)).compare("south") == 0) {
					bool stopRobot = false;
					while(1) {
						// check for north walls
						for(list<pair<int, int>>::iterator k = wallsNorth.begin(); k != wallsNorth.end(); k++) {
							if(k->first == x && k->second == y+1) stopRobot = true;
						}
						// check for other robots
						for(unsigned l=0; l<numRobots; l++) {
							if(l != j && get<0>(robotPositions[l].back()) == x && get<1>(robotPositions[l].back()) == y+1) stopRobot = true;
						}
						// check for end of board
						if(y+1 > height) stopRobot = true;
						// check for break
						if(!stopRobot) y += 1;
						else break;
					}
				} else if(get<2>(moveList.at(i)).compare("west") == 0) {
					bool stopRobot = false;
					while(1) {
						// check for west walls
						for(list<pair<int, int>>::iterator k = wallsWest.begin(); k != wallsWest.end(); k++) {
							if(k->first == x && k->second == y) stopRobot = true;
						}
						// check for other robots
						for(unsigned l=0; l<numRobots; l++) {
							if(l != j && get<0>(robotPositions[l].back()) == x-1 && get<1>(robotPositions[l].back()) == y) stopRobot = true;
						}
						// check for end of board
						if(x-1 < 1) stopRobot = true;
						// check for break
						if(!stopRobot) x -= 1;
						else break;
					}
				}
				tempPositions[j].push_back(pair<int,int>(x,y));
			}
		}
		for(unsigned j=0; j<numRobots; j++) {
			robotPositions[j].push_back(pair<int,int>(get<0>(tempPositions[j].back()),get<1>(tempPositions[j].back())));
			//cout << tempPositions[j].size() << endl;
			//tempPositions[j].pop_back();
		}

	}
	return;
}

void RobotViz::initSDL() {
	atexit(SDL_Quit);
	// initialize 
	if(SDL_Init(SDL_INIT_VIDEO) < 0) exit(1);
	// set caption
	SDL_WM_SetCaption("Ricochet Robots Visualization", NULL);
	// create window
	screenWidth = width * tileSize;
	screenHeight = (height+1) * tileSize;
	origRatio = (double)screenWidth / (double)screenHeight;
	currRatio = origRatio;
	screen = SDL_SetVideoMode(screenWidth, screenHeight, 32, SDL_HWSURFACE|SDL_RESIZABLE);
	// check, if initialization failed
	if(screen == NULL) {
		cerr << "Error: SDL could not be initialized!" << endl;
		exit(1);
	}
	board = SDL_CreateRGBSurface(0,(width*tileSize),((height+1)*tileSize),32,0,0,0,0);
	return;
}

string RobotViz::getPath() {
	char buff[1024];
	ssize_t len = readlink("/proc/self/exe", buff, sizeof(buff)-1);
	if (len != -1) {
		buff[len] = '\0';
		string path = string(buff);
		path = path.substr(0,path.find_last_of("\\/"));
		return path;
	} else {
		cerr << "Error: Could not determine binary path!" << endl;
		exit(1);
	}
}

void RobotViz::setupImages() {
	string path = getPath();
	// tiles
	tile[tile_empty] = IMG_Load((path + "/robotviz-img/tile_empty.png").c_str());
	tile[tile_left]  = IMG_Load((path + "/robotviz-img/tile_left.png").c_str());
	tile[tile_right] = IMG_Load((path + "/robotviz-img/tile_right.png").c_str());
	// walls
	robot[robot_red]    = IMG_Load((path + "/robotviz-img/robot_red.png").c_str());
	robot[robot_green]  = IMG_Load((path + "/robotviz-img/robot_green.png").c_str());
	robot[robot_blue]   = IMG_Load((path + "/robotviz-img/robot_blue.png").c_str());
	robot[robot_yellow] = IMG_Load((path + "/robotviz-img/robot_yellow.png").c_str());
	// walls
	wallNorth = IMG_Load((path + "/robotviz-img/wall_north.png").c_str());
	wallWest  = IMG_Load((path + "/robotviz-img/wall_west.png").c_str());
	// items
	item[sun_red]        = IMG_Load((path + "/robotviz-img/sun_red.png").c_str());
	item[sun_green]      = IMG_Load((path + "/robotviz-img/sun_green.png").c_str());
	item[sun_blue]       = IMG_Load((path + "/robotviz-img/sun_blue.png").c_str());
	item[sun_yellow]     = IMG_Load((path + "/robotviz-img/sun_yellow.png").c_str());
	item[moon_red]       = IMG_Load((path + "/robotviz-img/moon_red.png").c_str());
	item[moon_green]     = IMG_Load((path + "/robotviz-img/moon_green.png").c_str());
	item[moon_blue]      = IMG_Load((path + "/robotviz-img/moon_blue.png").c_str());
	item[moon_yellow]    = IMG_Load((path + "/robotviz-img/moon_yellow.png").c_str());
	item[star_red]       = IMG_Load((path + "/robotviz-img/star_red.png").c_str());
	item[star_green]     = IMG_Load((path + "/robotviz-img/star_green.png").c_str());
	item[star_blue]      = IMG_Load((path + "/robotviz-img/star_blue.png").c_str());
	item[star_yellow]    = IMG_Load((path + "/robotviz-img/star_yellow.png").c_str());
	item[planet_red]     = IMG_Load((path + "/robotviz-img/planet_red.png").c_str());
	item[planet_green]   = IMG_Load((path + "/robotviz-img/planet_green.png").c_str());
	item[planet_blue]    = IMG_Load((path + "/robotviz-img/planet_blue.png").c_str());
	item[planet_yellow]  = IMG_Load((path + "/robotviz-img/planet_yellow.png").c_str());
	item[random_robot]   = IMG_Load((path + "/robotviz-img/random_robot.png").c_str());
	// arrows
	arrow[arrow_red_north]    = IMG_Load((path + "/robotviz-img/arrow_red.png").c_str());
	arrow[arrow_green_north]  = IMG_Load((path + "/robotviz-img/arrow_green.png").c_str());
	arrow[arrow_blue_north]   = IMG_Load((path + "/robotviz-img/arrow_blue.png").c_str());
	arrow[arrow_yellow_north] = IMG_Load((path + "/robotviz-img/arrow_yellow.png").c_str());
	for (int i = 1; i != 4; ++i) {
		for (int j = 0; j != 4; ++j) {
			arrow[i*4 + j] = rotateSurface90Degrees(arrow[j], i);
		}
	}
	return;
}

void RobotViz::pollEvents(){
	SDL_Event event;

	while(SDL_PollEvent(&event)){
		switch(event.type){
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym){
					case SDLK_ESCAPE:
						exit(0);
					case SDLK_RIGHT:
						if(step < moveList.size()) step++;
						break;
					case SDLK_LEFT:
						if(step > 0) step--;
						break;
					case SDLK_HOME:
						step = 0;
						break;
					case SDLK_END:
						step = moveList.size();
						break;
					default:
						break;
				}
				break;
			case SDL_VIDEORESIZE:
				screenWidth = event.resize.w;
				screenHeight = event.resize.h;
				screen = SDL_SetVideoMode(screenWidth, screenHeight, 32, SDL_HWSURFACE|SDL_RESIZABLE);
				// check, if initialization failed
				if(screen == NULL) {
					cerr << "Error: SDL could not be initialized!" << endl;
					exit(1);
				}
				currRatio = (double)event.resize.w / (double)event.resize.h;
				if(currRatio > origRatio) {
					wOffset = (int)(((double)event.resize.w - (origRatio * (double)event.resize.h)) / 2);
					hOffset = 0;
				} else if(currRatio < origRatio) {
					wOffset = 0;
					hOffset = (int)(((double)event.resize.h - ((1 / origRatio) * (double)event.resize.w)) / 2);
				} else {
					wOffset = 0;
					hOffset = 0;
				}
				break;
			case SDL_QUIT:
				exit(0);
			default:
				break;
		}
	}
}

void RobotViz::draw() {
	// clean
	Uint32 color = SDL_MapRGB(board->format, 255, 255, 255);
	SDL_FillRect(board, 0, color);
	// tiles
	for(int y = 0; y < height; y++){
		for(int x = 0; x < width; x++){
			SDL_Rect rect = {Uint16(x * tileSize), Uint16(y * tileSize), Uint16(tileSize), Uint16(tileSize)};
			if((x+y) % 2) SDL_BlitSurface(tile[1], 0, board, &rect);
			else SDL_BlitSurface(tile[2], 0, board, &rect);
		}
	}
	// walls (north)
	for(list<pair<int, int>>::iterator i = wallsNorth.begin(); i != wallsNorth.end(); i++){
		SDL_Rect rect = {Uint16(tileSize * i->first - 6 - tileSize), Uint16(tileSize * i->second - 6 - tileSize), 62, 12};
		SDL_BlitSurface(wallNorth, 0, board, &rect);
	}
	// walls (west)
	for(list<pair<int, int>>::iterator i = wallsWest.begin(); i != wallsWest.end(); i++){
		SDL_Rect rect = {Uint16(tileSize * i->first - 6 - tileSize), Uint16(tileSize * i->second - 6 - tileSize), 62, 12};
		SDL_BlitSurface(wallWest, 0, board, &rect);
	}
	if(!boardOnly) {
		// target
		SDL_Rect rect = {Uint16(tileSize * (targetItem.x - 1)), Uint16(tileSize * (targetItem.y - 1)), Uint16(tileSize), Uint16(tileSize)};
		SDL_BlitSurface(item[targetItem.symbol], 0, board, &rect);
		// robots
		for(int i = 0; i < numRobots; i++){
			if( robotPositions[i].size() != 0 ) {
				SDL_Rect rect = {Uint16(robotPositions[i].at(step).first * tileSize - tileSize), Uint16(robotPositions[i].at(step).second * tileSize - tileSize), Uint16(tileSize), Uint16(tileSize)};
				SDL_BlitSurface(robot[i], 0, board, &rect);
				//cout << i << ":" << robotPositions[i].at(step).first << "," << robotPositions[i].at(step).second << "(" << step << ")" << endl;
			}
		}
		// arrows
		if(step < moveList.size()) {
			int pos = 0;
			for(int i = step; i < moveList.size(); i++) {
				SDL_Rect rect = {Uint16(pos*tileSize), Uint16(height*tileSize), Uint16(tileSize), Uint16(tileSize)};
				SDL_BlitSurface(arrow[get<1>(moveList.at(i))], 0, board, &rect);
				pos++;
			}
		}
	}
	// draw
	SDL_Rect rect;
	double zoom;
	bool free = false;
	SDL_Surface *zSurface;
	if(currRatio > origRatio) {
		if(screenHeight != tileSize*(height+1)) {
			zoom = (double)screenHeight / ((double)tileSize*(double)(height+1));
			zSurface = zoomSurface(board, zoom, zoom, 0);
			free = true;
		}
	} else if(currRatio < origRatio) {
		if(screenWidth != tileSize*width) {
			zoom = (double)screenWidth / ((double)tileSize*(double)width);
			zSurface = zoomSurface(board, zoom, zoom, 0);
			free = true;
		}
	} else {
		if(screenWidth != tileSize*width) {
			zoom = (double)screenWidth / ((double)tileSize*(double)width);
			zSurface = zoomSurface(board, zoom, zoom, 0);
			free = true;
		}
	}
	rect = {Uint16(wOffset), Uint16(hOffset), Uint16(wOffset + tileSize*width), Uint16(wOffset + tileSize*(height+1))};
	if(free) {
		SDL_BlitSurface(zSurface, NULL, screen, &rect);
		SDL_FreeSurface(zSurface);
	} else {
		SDL_BlitSurface(board, NULL, screen, &rect);
	}
	SDL_Flip(screen);
	return;
}

int main(int argc, char **argv) {

	cout << "robotviz 0.1" << endl;
	cout << "Copyright (C) Holger Jost" << endl;
	cout << "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>" << endl;
	cout << "robotviz is free software: you are free to change and redistribute it." << endl;
	cout << "There is NO WARRANTY, to the extent permitted by law." << endl;

	string input;
	string lineInput;

	bool beginParsing = false;
	bool stopParsing = false;

	/* write input to string */
	while(getline(cin,lineInput)) {
		if(!beginParsing) {
			if(lineInput.compare("Answer: 1") == 0) beginParsing = true;
		} else {
			if(lineInput.compare("SATISFIABLE") == 0 || lineInput.compare("Answer: 2") == 0) {
				stopParsing = true;
			}
			if(!stopParsing && lineInput.substr(0,1).compare("%") != 0) {
				input += lineInput + "\n";
			}
		}
	}

	cout << input << endl;

	srand(time(0));

	RobotViz::parseDim(input);
	RobotViz::parseBarrier(input);
	RobotViz::parsePosition(input);
	RobotViz::parseTarget(input);
	RobotViz::parseMove(input);

	RobotViz::constructBoard();
	RobotViz::calcPositions();

	RobotViz::initSDL();
	RobotViz::setupImages();

	while(1){
		RobotViz::pollEvents();
		RobotViz::draw();
		SDL_Delay(50);
	}

	return 0;
}
